// waifusearch, the IB searcher
// ============================
// -This software is designed to search an imageboard's text.

// -Filename: Timer.hpp
// -Implementation of the Timer class

#pragma once

#include <chrono>
#include <string>
#include <utility>
#include <vector>

#include "user_flags.hpp"

namespace rw {

// alias'
using Clock   = std::chrono::steady_clock;
using Time_pt = std::chrono::time_point<Clock>;

// globals
static std::vector<std::pair<Time_pt, std::string>> named_timings{};
//
bool do_timing{true};

// usings
using std::make_pair;
using std::move;
using std::string;

/**-----------------------------------------------------------------------------
 @brief    This is an easy to use named-timer mechanism. Whenever a Timer object
 is instantiated, it stores it's start time automatically. When it goes out of
 scope, or is explicitly stopped, it calculates the difference and stores that
 time into the system container 'named_timings' (declared above).

 @param    timer_name [IN] The name for the timer.
 @func     stop() Stop the timer.
*/
class Timer {
 public:
  Timer(const std::string& timer_name)
      : start{Clock::now()}, name{move(timer_name)} {}
  ~Timer() {
    if (! have_stop)
      stop();
  }

  void stop() {
    if (do_timing)
      named_timings.emplace_back(make_pair((Clock::now() - start), move(name)));
    have_stop = true;
  }

 private:
  const Time_pt start;
  const string  name;
  bool          have_stop{false};
};

}  // namespace rw

// Copyright (2021)
// License (MIT) https://opensource.org/licenses/MIT
